#!/bin/sh

die() {
    echo $1 >&2
    exit 1
}

findcmd() {
    type "$1" &> /dev/null || \
        die "Couldn't find $1, please install ${2:-it} or fix your path."
}

olsrd_pidfile=/var/run/ahcp_olsrd_pid
ariadne_pidfile=/var/run/ahcp_ariadne_pid
usage="Usage: $0 (start|stop)"
debuglevel=${AHCP_DEBUG_LEVEL:-1}

[ $# -eq 1 ] || die "$usage"

if [ $debuglevel -ge 1 ]; then
   env | grep AHCP
fi

if [ $debuglevel -ge 2 ]; then
   set -x
fi

interface="$AHCP_INTERFACE"
if [ -z "$interface" ]; then die "No interface set"; fi

findcmd ip iproute

(ip -6 addr show dev lo | grep -q 'inet6') || \
    die "No IPv6 address on lo, please modprobe ipv6"

l="$(ip -0 addr show dev "$interface" | grep '^ *link/ether ' | head -n 1)"
mac="$(echo "$l" | sed 's|^ *link/ether \([0-9a-z:]*\) .*$|\1|')"
prefix="$AHCP_PREFIX"
address="$(ahcp-generate-address $prefix $mac)"

nameserver_start() {
    if [ ! -z "$AHCP_NAMESERVER" ]; then
        mv /etc/resolv.conf /etc/resolv.conf.orig
        for n in $AHCP_NAMESERVER; do
            echo "nameserver $n" >> /etc/resolv.conf
        done
    fi
}

nameserver_stop() {
    if [ ! -z "$AHCP_NAMESERVER" ]; then
        mv /etc/resolv.conf.orig /etc/resolv.conf
    fi
}    

start_none() {
    ip -6 addr add "$address" dev "$interface"
}

stop_none() {
    ip -6 addr del "$address" dev "$interface"
}

start_static() {
    ip -6 addr add "$address" dev "$interface"
    for gw in $AHCP_STATIC_DEFAULT_GATEWAY; do
        ip -6 route add ::/0 via $gw
    done
}

stop_static() {
    for gw in $AHCP_STATIC_DEFAULT_GATEWAY; do
        ip -6 route del ::/0 via $gw
    done
    ip -6 addr del "$address" dev "$interface"
}

start_olsr() {
    multicast="$AHCP_OLSR_MULTICAST_ADDRESS"

    conf_filename=/var/run/ahcp-olsrd-"$AHCP_DAEMON_PID".conf

    cat > "$conf_filename" <<EOF
DebugLevel $debuglevel

IpVersion 6

AllowNoInt yes

Interface "$interface" {
    Ip6AddrType global
    Ip6MulticastSite $multicast
    Ip6MulticastGlobal $multicast
}
EOF
    
    if [ "${AHCP_OLSR_LINK_QUALITY:-0}" -ge 1 ]; then
        echo "UseHysteresis no" >> "$conf_filename"
        echo "LinkQualityLevel $AHCP_OLSR_LINK_QUALITY" >> "$conf_filename"
    fi

    for file in "/usr/local/lib/ahcp/ahcp-olsrd-$(uname -n).conf" \
                "/usr/local/lib/ahcp/ahcp-olsrd.conf" ; do
        if [ -r "$file" ]; then
            sed "s/\$AHCP_OLSR_MULTICAST_ADDRESS/$multicast/" < "$file" \
                >> "$conf_filename"
        fi
    done

    ip -6 addr add "$address" dev "$interface" || die "Couldn't add address"
    nameserver_start
    olsrd -f "$conf_filename" -nofork &
    echo $! > $olsrd_pidfile
}

stop_olsr() {
    conf_filename=/var/run/ahcp-olsrd-"$AHCP_DAEMON_PID".conf

    kill $(cat "$olsrd_pidfile")
    rm "$olsrd_pidfile"

    ip -6 addr del "$address" dev "$interface"
    rm "$conf_filename"
    nameserver_stop
}

start_ariadne() {
    multicast="${AHCP_ARIADNE_MULTICAST_ADDRESS:+-m $AHCP_ARIADNE_MULTICAST_ADDRESS}"
    port="${AHCP_ARIADNE_PORT_NUMBER:+-p $AHCP_ARIADNE_PORT_NUMBER}"
    hello="${AHCP_ARIADNE_HELLO_INTERVAL:+-h $AHCP_ARIADNE_HELLO_INTERVAL}"

    if [ -r /usr/local/lib/ahcp/ahcp-options ] ; then
        options="$(cat /usr/local/lib/ahcp/ahcp-options)"
    fi

    if [ -r /usr/local/lib/ahcp/ahcp-interfaces ] ; then
        interfaces="$(cat /usr/local/lib/ahcp/ahcp-interfaces)"
    fi

    ip -6 addr add "$address" dev "$interface" || die "Couldn't add address"
    nameserver_start
    
    ariadne -d $debuglevel $multicast $port $hello $options $address $interface $interfaces &
    echo $! > $ariadne_pidfile
}

stop_ariadne() {
    kill $(cat "$ariadne_pidfile")
    rm "$ariadne_pidfile"

    ip -6 addr del "$address" dev "$interface"
    nameserver_stop
}

case $1 in
    start)
        case ${AHCP_ROUTING_PROTOCOL:-none} in
            none) start_none;;
            static) start_static;;
            OLSR) start_olsr;;
            [Aa]riadne) start_ariadne;;
            *) die "Unknown routing protocol $AHCP_ROUTING_PROTOCOL";;
        esac;;
    stop)
        case ${AHCP_ROUTING_PROTOCOL:-none} in
            none) stop_none;;
            static) stop_static;;
            OLSR) stop_olsr;;
            [Aa]riadne) stop_ariadne;;
            *) die "Unknown routing protocol $AHCP_ROUTING_PROTOCOL";;
        esac;;
esac
